abstract RoboWorld =
	RoboWorldLexicon,
	Numeral
	**
{

	--------------------------------------------------------------------------------
	cat -- closed categories
	
		Unit ;
		
	--------------------------------------------------------------------------------
	fun -- functions of closed categories
	
		m_Unit : Unit ;		
		mm_Unit : Unit ;
		ms_Unit : Unit ;
		minutes_Unit : Unit;
		rads_Unit : Unit ;
		s_Unit : Unit ;
		
	--------------------------------------------------------------------------------
	cat -- ItemPhrase
	
		BasicItem ;
		CompoundItem ;
		Item ;
		ItemPhrase ;
		ItemPhraseList ;
		
	--------------------------------------------------------------------------------
	fun -- ItemPhrase
	
		-- velocity
		mkBasicItem_single_noun : Cat.N -> BasicItem ;
		
		-- odometer value
		mkBasicItem_two_nouns : Cat.N -> Cat.N -> BasicItem ;		
		
		-- angular velocity
		mkBasicItem_QualifiedBI : Cat.A -> BasicItem -> BasicItem ;
		
		-- m/s
		mkBasicItem_Unit : Unit -> BasicItem ;
		
		-- m/s upwards
		mkCompoundItem_AdverbCI : Item -> Adv -> CompoundItem ;

		-- object initially
		mkCompoundItem_AdverbCI_from_adjective : Item -> A -> CompoundItem ;
		
		-- distance from the robot to the nest
		mkCompoundItem_PrepositionCI_single_ItemPhrase : BasicItem -> Prep -> ItemPhrase -> CompoundItem ;
		
		-- location except the source and the nest
		mkCompoundItem_PrepositionCI_and_list_of_ItemPhrases : BasicItem -> Prep -> ItemPhraseList -> CompoundItem ;
		
		-- angular velocity
		mkItem_from_BasicItem : BasicItem -> Item ;
		
		-- angular velocity of the robot
		mkItem_from_CompoundItem : CompoundItem -> Item ;
		
		-- it
		mkItemPhrase_PronounIP : Cat.Pron -> ItemPhrase ;
		
		-- the angular velocity
		mkItemPhrase_DeterminedIP : Cat.Det -> Item -> ItemPhrase ;
		
		-- 1 position
		mkItemPhrase_QuantifiedIP_with_digits : Cat.Digits -> Item -> ItemPhrase ;
		
		-- one position
		mkItemPhrase_QuantifiedIP_with_text : Cat.Numeral -> Item -> ItemPhrase ;
		
		-- 0.5 m
		mkItemPhrase_QuantifiedIP_with_float : Float -> Item -> ItemPhrase ;
		
		-- at least 1 position
		mkItemPhrase_AdN_QuantifiedIP_with_digits : AdN -> Cat.Digits -> Item -> ItemPhrase ;
		
		-- at least one position
		mkItemPhrase_AdN_QuantifiedIP_with_text : AdN -> Cat.Numeral -> Item -> ItemPhrase ;
		
		-- at least 0.5 m
		mkItemPhrase_AdN_QuantifiedIP_with_float : AdN -> Float -> Item -> ItemPhrase ;		
		
		-- no obstacles
		mkItemPhrase_QuantifiedIP_with_Quant : Cat.Quant -> Item -> ItemPhrase ;
		
		-- 0.0
		mkItemPhrase_Float_Literal : Float -> ItemPhrase ;
		
		-- [an x-width of 1 m, an y-width of 1 m]
		mkItemPhraseList_binary : ItemPhrase -> ItemPhrase -> ItemPhraseList ;

		-- [an x-width of 1 m, an y-width of 1 m, an z-width of 1 m]
		mkItemPhraseList_many : ItemPhrase -> ItemPhraseList -> ItemPhraseList ;
		


	--------------------------------------------------------------------------------
	cat -- Statement
	
		Statement ;
		StatementList ;
		Statements ;
		
	--------------------------------------------------------------------------------
	fun -- Statement		
	
		-- the odometer of the robot is reset
		mkStatement_PassiveVoice_IntransitiveVerb : ItemPhrase -> V -> Statement ;	
	
		-- the velocity of the robot is set to 1 m/s upward
		mkStatement_PassiveVoice_TransitiveVerb_Preposition_ItemPhrase : ItemPhrase -> V2 -> Prep -> ItemPhrase -> Statement ;	
		
		-- the operation takeOff was called in 20 minutes before
		mkStatement_PastTense_PassiveVoice_TransitiveVerb_Preposition_ItemPhrase : ItemPhrase -> V -> Prep -> ItemPhrase -> Statement ;		
		
		-- the arena is three-dimensional
		mkStatement_ActiveVoice_ToBe_Adjective : ItemPhrase -> A -> Statement ;

		-- the tank of water is either full or empty
		mkStatement_ActiveVoice_ToBe_Conj_Adjective_Adjective : ItemPhrase -> Conj -> A -> A -> Statement ;	
				
		-- the gradient of the ground is 0.0
		mkStatement_ActiveVoce_ToBe_ItemPhrase : ItemPhrase -> ItemPhrase -> Statement ;
		
		-- the robot is in the origin initially
		mkStatement_ActiveVoice_ToBe_Preposition_ItemPhrase : ItemPhrase -> Prep -> ItemPhrase -> Statement ;

		-- initially the robot is in the origin
		mkStatement_ActiveVoice_AdverbFromAdjective_ToBe_Preposition_ItemPhrase : A -> ItemPhrase -> Prep -> ItemPhrase -> Statement ;
		
		-- the distance from the target to the origin is greater than 1 m
		mkStatement_ActiveVoice_ToBe_Positive_Comparison_ItemPhrase : ItemPhrase -> A -> ItemPhrase -> Statement ;	
	
		-- the distance from the target to the origin is not greater than 1 m
		mkStatement_ActiveVoice_ToBe_Negative_Comparison_ItemPhrase : ItemPhrase -> A -> ItemPhrase -> Statement ;		
	
		-- the robot places an object in the nest
		mkStatement_ActiveVoice_TransitiveVerb_DirectObject : ItemPhrase -> V2 -> ItemPhrase -> Statement ;

		-- the event spray occurred in 3 minutes before
		mkStatement_PastTense_ActiveVoice_TransitiveVerb_Preposition_ItemPhrase : ItemPhrase -> V -> Prep -> ItemPhrase -> Statement ;

		-- the robot may carry 1 objects
		mkStatement_ActiveVoice_Modal_TransitiveVerb_DirectObject : ItemPhrase -> VV -> V2 -> ItemPhrase -> Statement ;

		-- the nest may contain up to 5 objects
		mkStatement_ActiveVoice_Modal_TransitiveVerb_Prep_DirectObject : ItemPhrase -> VV -> V2 -> Prep -> ItemPhrase -> Statement ;

		-- it is raining
		mkStatement_ActiveVoice_Positive_Progressive_IntransitiveVerb : ItemPhrase -> V -> Statement ;

		-- it is not raining
		mkStatement_ActiveVoice_Negative_Progressive_IntransitiveVerb : ItemPhrase -> V -> Statement ;	

		-- the robot is carrying an object
		mkStatement_ActiveVoice_Positive_Progressive_TransitiveVerb_DirectObject : ItemPhrase -> V2 -> ItemPhrase -> Statement ;

		-- the robot is not carrying an object
		mkStatement_ActiveVoice_Negative_Progressive_TransitiveVerb_DirectObject : ItemPhrase -> V2 -> ItemPhrase -> Statement ;	

		-- [the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward]
		mkStatementList_binary : Statement -> Statement -> StatementList ;

		-- [the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward, the angular velocity of the robot is set to 1 rad/s]
		mkStatementList_many : Statement -> StatementList -> StatementList ;

		-- the velocity of the robot is set to 1 m/s upward
		mkStatements_single_statement : Statement -> Statements ;

		-- the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward and the angular velocity of the robot is set to 1 rad/s
		mkStatements_list_of_statements : StatementList -> Statements ;
		
		-- the event spray occurred in 3 minutes before or the operation takeOff was called in 20 minutes before
		mkStatements_or_list_of_statements : StatementList -> Statements ;		

	--------------------------------------------------------------------------------
	cat -- Conditions
	
		Conditions ;
		
	--------------------------------------------------------------------------------
	fun -- Conditions		
		
		-- when the distance from the robot to the source is less than 1 m,
		-- the distance from the robot to the nest is more than 2 m and the robot is carrying an object
		mkConditions_Subj_Statements : Subj -> Statements -> Conditions ;

	--------------------------------------------------------------------------------
	cat -- ArenaAssumption
	
		ArenaAssumption ;
		
	--------------------------------------------------------------------------------
	fun -- ArenaAssumption
	
		-- some locations of the arena except the source and the nest contain 1 obstacles
		mkArenaAssumption_Statement : Statement -> ArenaAssumption ;



	--------------------------------------------------------------------------------
	cat -- RobotAssumption
	
		RobotAssumption ;
		
	--------------------------------------------------------------------------------
	fun -- RobotAssumption
	
		-- the robot is a point mass
		mkRobotAssumption_RobotStatement : Statement -> RobotAssumption ;



	--------------------------------------------------------------------------------
	cat -- ElementAssumption
	
		ElementAssumption ;
		
	--------------------------------------------------------------------------------
	fun -- ElementAssumption
	
		-- the source has an x-width of 0.25 m and a y-width of 0.25 m
		mkElementAssumption_Statement : Statement -> ElementAssumption ;
		

		
	--------------------------------------------------------------------------------
	cat -- InputEventMapping
	
		InputEventMapping ;
		
	--------------------------------------------------------------------------------
	fun -- InputEventMapping
	
		-- the event obstacle occurs when the distance from the robot to an obstacle is less than 1 m
		mkInputEventMapping_InputSometimes : String -> Conditions -> InputEventMapping ;

		-- the event angularSpeed is always available and it communicates the angular velocity of the robot
		mkInputEventMapping_InputAlways : String -> Statements -> InputEventMapping ;

		-- the event transferred never happens
		mkInputEventMapping_InputNever : String -> InputEventMapping ;

	--------------------------------------------------------------------------------
	cat -- OutputEventMapping
	
		OutputEventMapping ;
		
	--------------------------------------------------------------------------------
	fun -- OutputEventMapping
	
		-- when the event takeoff occurs the velocity of the robot is set to 1 m/s upward
		mkOutputEventMapping_OutputSometimes_NoConditions : String -> Statements -> OutputEventMapping ;
		
		-- the output event spray is defined by a diagram where one time unit is 1 s
		mkOutputEventMapping_DiagramaticOutput : String -> Float -> Unit -> OutputEventMapping ;
		
	--------------------------------------------------------------------------------
	cat -- OperationMapping
	
		OperationMapping ;
		
	--------------------------------------------------------------------------------
	fun -- OperationMapping
		
		-- when the operation move(ls,as) is called 
		-- the velocity of the robot is set to ls m/s towards the orientation of the robot
		-- and the angular velocity of the robot is set to as rad/s		
		mkOperationMapping_OutputSometimes_NoConditions : String -> Statements -> OperationMapping ;
		
		-- when the operation Store() is called as soon as the distance from the robot to the source is less than 1 m the robot places an object in the nest
		mkOperationMapping_OutputSometimes_WithConditions : String -> Conditions -> Statements -> OperationMapping ;		
		
		-- when the operation Transfer() is called nothing happens
		mkOperationMapping_NoOutput : String -> OperationMapping ;

		-- the operation turnBack() is defined by a diagram where one time unit is 1 s
		mkOperationMapping_DiagrammaticOutput : String -> Float -> Unit -> OperationMapping ;			
		
	--------------------------------------------------------------------------------
	cat -- VariableMapping
	
		VariableMapping ;
		
	--------------------------------------------------------------------------------
	--fun -- VariableMapping



	--------------------------------------------------------------------------------
	-- SPECIAL WORDS (HELP FUNCTION)
	fun _special_N : N;
	fun _special_A : A;
	fun _special_Adv : Adv;
	fun _special_Prep : Prep;
	fun _special_Pron : Pron;
	--fun _special_Det : Det;
	fun _special_V2 : V2;
	fun _special_Subj : Subj;
	
	fun _special_empty_V : V;
	fun _special_Unit : Unit;
	fun _special_BasicItem : BasicItem;
	fun _special_CompoundItem : CompoundItem;
	fun _special_Item : Item;
	fun _special_ItemPhrase : ItemPhrase;
	fun _special_ItemPhraseList : ItemPhraseList;	
	fun _special_Statement : Statement;
	fun _special_StatementList : StatementList;
	fun _special_Statements : Statements;	
	fun _special_Conditions : Conditions;
	fun _special_ArenaAssumption : ArenaAssumption;			
	fun _special_RobotAssumption : RobotAssumption;
	fun _special_ElementAssumption : ElementAssumption;
	fun _special_InputEventMapping : InputEventMapping;
	fun _special_OutputEventMapping : OutputEventMapping;
	fun _special_OperationMapping : OperationMapping;
	fun _special_VariableMapping : VariableMapping;	
		
}
