concrete RoboWorldEng of RoboWorld =
	RoboWorldLexiconEng,
	NumeralEng
	**
open
	SyntaxEng,
	(ResEng = ResEng),
	ParadigmsEng,
	SymbolicEng,
	ExtraEng,
	Prelude,
	MorphoEng,
	ParamX
in {

	--------------------------------------------------------------------------------
	lincat -- closed categories
	
		Unit = CatEng.N ;

	--------------------------------------------------------------------------------
	lin -- functions of closed categories
	
		m_Unit = mkN "m" "m" ;
		mm_Unit = mkN "mm" "mm" ;		
		ms_Unit = mkN "m/s" "m/s" ;
		minutes_Unit = mkN "minute" "minutes" ;
		rads_Unit = mkN "rad/s" "rad/s" ;
		s_Unit = mkN "s" "s" ;

	--------------------------------------------------------------------------------
	lincat -- ItemPhrase
	
		BasicItem = CatEng.CN ;
		CompoundItem = CatEng.CN ;
		Item = CatEng.CN ;
		ItemPhrase = CatEng.NP ;
		ItemPhraseList = ListNP ;

	--------------------------------------------------------------------------------
	lin -- ItemPhrase
	
		mkBasicItem_single_noun n =
			-- mkCN : N -> CN | velocity
			mkCN
				(lin N n) ; -- velocity

		mkBasicItem_two_nouns n1 n2 =
			-- mkCN : N -> NP -> CN | odometer value
			mkCN
				(
					-- mkN : Str -> N -> N | odometer value
					mkN
						(n1.s ! ResEng.Sg ! ResEng.Nom) -- odometer
						(lin N n2) -- value
				) ;
								
		mkBasicItem_QualifiedBI adj cn =
			-- mkCN : A -> CN -> CN | angular velocity
			mkCN
				<lin A adj : A> -- angular
				<cn : CN> ; -- velocity

		mkBasicItem_Unit unit =
			-- mkCN : N -> CN | m/s
			mkCN
				unit ; -- m/s		
				
		mkCompoundItem_AdverbCI item adv =
			-- mkCN : CN -> Adv -> CN | m/s upwards
			mkCN
				<item : CN> -- m/s
				<lin Adv adv : Adv> ; -- upwards

		mkCompoundItem_AdverbCI_from_adjective item adj =
			-- mkCN : CN -> Adv -> CN | object initially
			mkCN
				item -- object
				(
					-- mkAdv : A -> Adv | initially
					SyntaxEng.mkAdv
						(lin A adj) -- initial
				) ;
				
		mkCompoundItem_PrepositionCI_single_ItemPhrase basic prep itemPhrase =
			-- mkCN : CN -> Adv -> CN | distance from the robot to the nest
			mkCN
				basic -- distance
				(
					-- mkAdv : Prep -> NP -> Adv | from the robot to the nest
					SyntaxEng.mkAdv
						(lin Prep prep) -- from
						itemPhrase -- the robot to the nest
				) ;

		mkCompoundItem_PrepositionCI_and_list_of_ItemPhrases basic prep list =
			-- mkCN : CN -> Adv -> CN | location except the source and the nest
			mkCN
				basic -- location
				(
					-- mkAdv : Prep -> NP -> Adv | except the source and the nest
					SyntaxEng.mkAdv
						(lin Prep prep) -- except
						(
							-- mkNP : Conj -> ListNP -> NP | the source and the nest
							mkNP
								and_Conj -- and
								list -- [the source, the nest]
						)
				) ;				

		mkItem_from_BasicItem basicItem =
			basicItem ; -- angular velocity
				
		mkItem_from_CompoundItem compoundItem =
			compoundItem ; -- angular velocity of the robot
			
		-- it
		mkItemPhrase_PronounIP pron =
			-- mkNP : Pron -> NP | it
			mkNP
				<lin Pron pron : Pron> ; -- it
		
		-- the angular velocity
		mkItemPhrase_DeterminedIP det item =		
			-- mkNP : Det -> CN -> NP | the angular velocity
			mkNP
				<lin Det det : Det> -- the
				<item : CN> ; -- angular velocity	
		
		-- 1 position
		mkItemPhrase_QuantifiedIP_with_digits digits item =
			-- mkNP : Det -> CN -> NP | 1 position
			mkNP
				(mkDet <(lin Digits digits) : Digits>) -- 1
				item ; -- position
				
		-- one position
		mkItemPhrase_QuantifiedIP_with_text numeral item =
			-- mkNP : Det -> CN -> NP | one position
			mkNP
				(mkDet <(lin Numeral numeral) : Numeral>) -- one
				item ; -- position		
				
		-- 0.5 m
		mkItemPhrase_QuantifiedIP_with_float float item =
		    let
		    	sym : Symb = mkSymb float.s ; -- mkSymb : Str -> Symb | 0.5
        		card : Card = symb sym ; -- symb : Symb -> Card | 0.5
		        det : Det = mkDet card ; -- mkDet : Card -> Det | 0.5
		     in
		     	mkNP
		     		det -- 0.5
		     		item ; -- m   
		     		
		-- at least 1 position
		mkItemPhrase_AdN_QuantifiedIP_with_digits adn digits item =
			-- mkNP : Det -> CN -> NP | at least 1 position
			mkNP
				(
					-- mkDet : Card -> Det | at least 1
					mkDet
					(
						-- mkCard : AdN -> Card -> Card | at least 1
						mkCard
							<(lin AdN adn) : AdN> -- at least
							(
								-- mkCard : Digits -> Card | 1
								mkCard <(lin Digits digits) : Digits> -- 1
							)
					)
				)
				item ; -- position
				
		-- at least one position
		mkItemPhrase_AdN_QuantifiedIP_with_text adn numeral item =
			-- mkNP : Det -> CN -> NP | at least one position
			mkNP
				(
					-- mkDet : Card -> Det | at least one
					mkDet
					(
						-- mkCard : AdN -> Card -> Card | at least one
						mkCard
							<(lin AdN adn) : AdN> -- at least
							(
								-- mkCard : Numeral -> Card | one
								mkCard <(lin Numeral numeral) : Numeral> -- one
							)
					)
				)
				item ; -- position								

		-- at least 0.5 m
		mkItemPhrase_AdN_QuantifiedIP_with_float adn float item =
		    let
		    	sym : Symb = mkSymb float.s ; -- mkSymb : Str -> Symb | 0.5
        		card : Card = symb sym ; -- symb : Symb -> Card | 0.5
        		adn_card : Card = mkCard <(lin AdN adn) : AdN> card ; -- mkCard : AdN -> Card -> Card | at least 0.5
		        det : Det = mkDet adn_card ; -- mkDet : Card -> Det | 0.5
		     in
		     	mkNP
		     		det -- at least 0.5
		     		item ; -- m     		
		     		
		-- no obstacles
		mkItemPhrase_QuantifiedIP_with_Quant quant item =
			mkNP
				(
					mkDet <(lin Quant quant) : Quant> plNum -- no
				)
				item ; -- obstacles
				
		-- 0.0
		mkItemPhrase_Float_Literal float =
			-- symb : Float -> NP | 0.0
			symb float ; -- 0.0
				
		-- [an x-width of 1 m, a y-width of 1 m]
		mkItemPhraseList_binary itemPhrase1 itemPhrase2 =
			-- mkListNP : NP -> NP -> ListNP | [an x-width of 1 m, a y-width of 1 m]
			mkListNP
				itemPhrase1 -- an x-width of 1 m
				itemPhrase2 ; -- a y-width of 1 m

		-- [an x-width of 1 m, a y-width of 1 m, a z-width of 1 m]
		mkItemPhraseList_many itemPhrase itemPhraseList =
			-- mkListNP : NP -> ListNP -> ListNP | [an x-width of 1 m, a y-width of 1 m, a z-width of 1 m]
			mkListNP
				itemPhrase -- an x-width of 1 m
				itemPhraseList ; -- [a y-width of 1 m, a z-width of 1 m]				



	--------------------------------------------------------------------------------
	lincat -- Statement
	
		Statement = CatEng.S ;
		StatementList = ListS ;
		Statements = CatEng.S ;

	--------------------------------------------------------------------------------
	lin -- Statement
	
		mkStatement_PassiveVoice_IntransitiveVerb itemPhrase v =
			-- mkS : Cl -> S | the odometer of the robot is reset
			mkS
				(
					-- mkCl : NP -> VP -> Cl | the odometer of the robot is reset
					mkCl
						itemPhrase -- the odometer of the robot
						(
							-- passiveVP : V2 -> VP | is reset
							passiveVP								
								(
									-- mkV2 : V -> V2 | reset
									mkV2 <(lin V v) : V> -- reset
								)
						)
				) ;	

		mkStatement_PassiveVoice_TransitiveVerb_Preposition_ItemPhrase itemPhrase1 v2 prep itemPhrase2 =
			-- mkS : Cl -> S | the velocity of the robot is set to 1 m/s upward
			mkS
				(
					-- mkCl : NP -> VP -> Cl | the velocity of the robot is set to 1 m/s upward
					mkCl
						itemPhrase1 -- the velocity of the robot
						(
							-- mkVP : VP -> Adv -> VP
							mkVP
								(
									-- passiveVP : V2 -> VP | is set
									passiveVP
										<(lin V2 v2) : V2> -- set
								)
								(
									-- mkAdv : Prep -> NP -> Adv | to 1 m/s upward
									SyntaxEng.mkAdv
										(lin Prep prep) -- to
										itemPhrase2 -- 1 m/s upward
								)
						)
				) ;	
				
		mkStatement_PastTense_PassiveVoice_TransitiveVerb_Preposition_ItemPhrase itemPhrase1 v prep itemPhrase2 =
			-- mkS : Tense -> Cl -> S | the operation takeOff was called in 20 minutes before
			mkS
				pastTense
				(
					-- mkCl : NP -> VP -> Cl | the operation takeOff is called in 20 minutes before
					mkCl
						itemPhrase1 -- the operation takeOff
						(
							-- mkVP : VP -> Adv -> VP
							mkVP
								(
									-- passiveVP : V2 -> VP | is called
									passiveVP
										(
											-- mkV2 : V -> V2 | call
											mkV2 <(lin V v) : V> -- call
										)
								)
								(
									-- mkAdv : Prep -> NP -> Adv | in 20 minutes before
									SyntaxEng.mkAdv
										(lin Prep prep) -- in
										itemPhrase2 -- 20 minutes before
								)
						)
				) ;											

		mkStatement_ActiveVoice_ToBe_Adjective itemPhrase adj =
			-- mkS : Cl -> S | the arena is three-dimensional
			mkS
				(
					-- mkCl : NP -> A -> Cl | the arena is three-dimensional
					mkCl
						itemPhrase -- the arena
						(lin A adj) -- three-dimensional
				) ;		

		mkStatement_ActiveVoice_ToBe_Conj_Adjective_Adjective itemPhrase conj adj1 adj2 =
			-- mkS : Cl -> S | the tank of water is either full or empty
			mkS
				(
					-- mkCl : NP -> AP -> Cl | the tank of water is either full or empty
					mkCl
						itemPhrase -- the tank of water
						(
							-- mkAP : Conj -> AP -> AP -> AP | either full or empty
							mkAP
								<lin Conj conj : Conj> -- either ... or ...
								<lin AP (
									-- mkAP : A -> AP | full
									mkAP <lin A adj1 : A> -- full
								) : AP>
								<lin AP (
									-- mkAP : A -> AP | empty
									mkAP <lin A adj2 : A> -- empty
								) : AP>							
						)
				) ;	

		mkStatement_ActiveVoce_ToBe_ItemPhrase itemPhrase1 itemPhrase2 =
			-- mkS : Cl -> S | the gradient of the ground is 0.0
			mkS
				(
					-- mkCl : NP -> NP -> Cl | the gradient of the ground is 0.0
					mkCl
						itemPhrase1 -- the gradient of the ground
						itemPhrase2 -- 0.0
				) ;
				
		mkStatement_ActiveVoice_ToBe_Preposition_ItemPhrase itemPhrase1 prep itemPhrase2 =
			-- mkS : Cl -> S | the robot is at the origin initially
			mkS
				(
					-- mkCl : NP -> Adv -> Cl | the robot is at the origin initially
					mkCl
						itemPhrase1 -- the robot
						(
							-- mkAdv : Prep -> NP -> Adv | at the origin initially
							SyntaxEng.mkAdv
								(lin Prep prep) -- at 
								itemPhrase2 -- the origin initially
						)					
				) ;											
				
		mkStatement_ActiveVoice_AdverbFromAdjective_ToBe_Preposition_ItemPhrase a itemPhrase1 prep itemPhrase2 =
			-- mkS : Adv -> S -> S | initially the robot is in the origin
			mkS
				(
					SyntaxEng.mkAdv
						<(lin A a) : A>
				)
				(
					-- mkS : Cl -> S | the robot is in the origin
					mkS
						(
							-- mkCl : NP -> Adv -> Cl | the robot is in the origin
							mkCl
								itemPhrase1 -- the robot
								(
									-- mkAdv : Prep -> NP -> Adv | in the origin
									SyntaxEng.mkAdv
										(lin Prep prep) -- in 
										itemPhrase2 -- the origin
								)					
						)				
				);
				
		mkStatement_ActiveVoice_ToBe_Positive_Comparison_ItemPhrase itemPhrase1 adj itemPhrase2 =
			-- mkS : Cl -> S | the distance from the target to the origin is greater than 1 m
			mkS
			(
    			-- mkCl : NP -> AP -> Cl | the distance from the target to the origin is greater than 1 m
    			mkCl
    				itemPhrase1 -- the distance from the target to the origin
    				(
    					-- mkAP : A -> NP -> AP | greater than 1 m
    					mkAP
    						(lin A adj) -- great    					
    						itemPhrase2 -- 1 m
    				)
    		) ;	
    		
		mkStatement_ActiveVoice_ToBe_Negative_Comparison_ItemPhrase itemPhrase1 adj itemPhrase2 =
    		-- mkS : Pol -> Cl -> S | the distance from the target to the origin is not greater than 1 m
    		mkS
    			UncNeg -- not
				(
					-- mkCl : NP -> AP -> Cl | the distance from the target to the origin is greater than 1 m
					mkCl
						itemPhrase1 -- the distance from the target to the origin
						(
							-- mkAP : A -> NP -> AP | greater than 1 m
							mkAP
								(lin A adj) -- great    					
								itemPhrase2 -- 1 m
						)
				) ;
				
		mkStatement_ActiveVoice_TransitiveVerb_DirectObject itemPhrase1 v2 itemPhrase2 =
			-- mkS : Cl -> S | the robot places an object in the nest
			mkS
				(
				-- mkCl : NP -> VP -> Cl | the robot places an object in the nest
				mkCl
					itemPhrase1 -- the robot		
					(
						-- mkVP : V2 -> NP -> VP | places an object in the nest
						mkVP
							<(lin V2 v2) : V2> -- places
							itemPhrase2 -- an object in the nest
					)					
				) ;	
				
		mkStatement_PastTense_ActiveVoice_TransitiveVerb_Preposition_ItemPhrase itemPhrase1 v prep itemPhrase2 =
			-- mkS : Tense -> Cl -> S | the event spray occurred in 3 minutes before
			mkS
				pastTense
				(
				-- mkCl : NP -> VP -> Cl | the event spray occurs in 3 minutes before
				mkCl
					itemPhrase1 -- the event spray
					(
						-- mkVP : VP -> Adv -> VP | occurs in 3 minutes before
						mkVP
							(
								-- mkVP : V -> VP | occurs
								mkVP
									<(lin V v) : V> -- occur
							)
							(
								-- mkAdv : Prep -> NP -> Adv | in 3 minutes before
								SyntaxEng.mkAdv
									(lin Prep prep) -- in
									itemPhrase2 -- 3 minutes before
							)
					)					
				) ;																
				
		mkStatement_ActiveVoice_Modal_TransitiveVerb_DirectObject itemPhrase1 vv v2 itemPhrase2 =
			-- mkS : Cl -> S | the robot may carry 1 objects
			mkS
				(
					-- mkCl : NP -> VV -> VP -> Cl | the robot may carry 1 objects
					mkCl
						itemPhrase1 -- the robot
						<(lin VV vv) : VV> -- may
						(
							-- mkVP : V2 -> NP -> VP | carry 1 objects
							mkVP
								<(lin V2 v2) : V2> -- carry 
								itemPhrase2 -- 1 objects
						)
				) ;	
				
		mkStatement_ActiveVoice_Modal_TransitiveVerb_Prep_DirectObject itemPhrase1 vv v2 prep itemPhrase2 =
			-- mkS : Cl -> S | the nest may contain up to 5 objects
			mkS
				(
					-- mkCl : NP -> VV -> VP -> Cl | the nest may contain up to 5 objects
					mkCl
						itemPhrase1 -- the nest
						<(lin VV vv) : VV> -- may
						(
							-- mkVP : V2 -> NP -> VP | contain up to 5 objects
							mkVP
								<(lin V2 v2) : V2> -- contain
								(
									-- mkNP : Predet -> NP -> NP | up to 5 objects
									mkNP
										<(lin Predet (
											-- ss : Str -> SS | up to
											ss
												prep.s -- up to
										)) : Predet>
										itemPhrase2 -- 5 objects
								)
						)
				);								

    	mkStatement_ActiveVoice_Positive_Progressive_IntransitiveVerb itemPhrase v =
    		-- mkS : Cl -> S | it is raining
    		mkS	
	    		(
	    			-- mkCl : NP -> VP -> Cl | it is raining
		    		mkCl
		    			itemPhrase -- it
		    			(
		    				-- progressiveVP : VP -> VP | is raining
		    				progressiveVP		    					
		    					(
		    						-- mkVP : V -> VP | rain
		    						mkVP
		    							<(lin V v) : V> -- rain
		    					)
		    			)
		    	) ;
		    	
		mkStatement_ActiveVoice_Negative_Progressive_IntransitiveVerb itemPhrase v =
    		-- mkS : Pol -> Cl -> S | it is not raining
    		mkS
    			UncNeg -- not	    		
	    		(
	    			-- mkCl : NP -> VP -> Cl | it is raining
		    		mkCl
		    			itemPhrase -- it		    			
		    			(
		    				-- progressiveVP : VP -> VP | is raining
		    				progressiveVP		    					
		    					(
		    						-- mkVP : V -> VP | rain
		    						mkVP
		    							<(lin V v) : V> -- rain
		    					)
		    			)
		    	) ;			    	
				
    	mkStatement_ActiveVoice_Positive_Progressive_TransitiveVerb_DirectObject itemPhrase1 v2 itemPhrase2 =
    		-- mkS : Cl -> S | the robot is carrying an object
    		mkS	
	    		(
	    			-- mkCl : NP -> VP -> Cl | the robot is carrying an object
		    		mkCl
		    			itemPhrase1 -- the robot
		    			(
		    				-- progressiveVP : VP -> VP | is carrying an object
		    				progressiveVP		    					
		    					(
		    						-- mkVP : V2 -> NP -> VP | carry an object
		    						mkVP
		    							<(lin V2 v2) : V2> -- carry
		    							itemPhrase2 -- an object
		    					)
		    			)
		    	) ;

		mkStatement_ActiveVoice_Negative_Progressive_TransitiveVerb_DirectObject itemPhrase1 v2 itemPhrase2 =
    		-- mkS : Pol -> Cl -> S | the robot is not carrying an object
    		mkS
    			UncNeg -- not	    		
	    		(
	    			-- mkCl : NP -> VP -> Cl | the robot is carrying an object
		    		mkCl
		    			itemPhrase1 -- the robot		    			
		    			(
		    				-- progressiveVP : VP -> VP | is carrying an object
		    				progressiveVP		    					
		    					(
		    						-- mkVP : V2 -> NP -> VP | carry an object
		    						mkVP
		    							<(lin V2 v2) : V2> -- carry
		    							itemPhrase2 -- an object
		    					)
		    			)
		    	) ;						

		mkStatementList_binary statement1 statement2 =
			-- mkListS : S -> S -> ListS | [the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward]
			mkListS
				statement1 -- the odometer of the robot is reset 
				statement2 ; -- the velocity of the robot is set to 1 m/s upward 

		mkStatementList_many statement statementList =
			-- mkListS : S -> ListS -> ListS | [the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward,
			--                                  the angular velocity of the robot is set to 1 rad/s]
			mkListS
				statement -- the odometer of the robot is reset
				statementList ; -- [the velocity of the robot is set to 1 m/s upward, the angular velocity of the robot is set to 1 rad/s]

		mkStatements_single_statement statement =
			statement ; -- the velocity of the robot is set to 1 m/s upward

		mkStatements_list_of_statements statementList =
			-- mkS : Conj -> ListS -> S | the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward
			--                            and the angular velocity of the robot is set to 1 rad/s
			mkS
				and_Conj -- and
				statementList ; -- [the odometer of the robot is reset, the velocity of the robot is set to 1 m/s upward,
				                --  the angular velocity of the robot is set to 1 rad/s]
		
		mkStatements_or_list_of_statements statementList =
			-- mkS : Conj -> ListS -> S | the event spray occurred in 3 minutes before
			--                            or the operation takeOff was called in 20 minutes before
			mkS
				or_Conj -- or
				statementList ; -- [the event spray occurred in 3 minutes before,
				                --  the operation takeOff was called in 20 minutes before]

	--------------------------------------------------------------------------------
	lincat -- Conditions
	
		Conditions = CatEng.Adv ;

	--------------------------------------------------------------------------------
	lin -- Conditions

		mkConditions_Subj_Statements subj statements =
			-- mkAdv : Subj -> S -> Adv | when the distance from the robot to the source is less than 1 m,
			--                            the distance from the robot to the nest is more than 2 m and the robot is carrying an object
			SyntaxEng.mkAdv
				<(lin Subj subj) : Subj> -- when 
				<statements : S> ; -- the distance from the robot to the source is less than 1 m,
				             -- the distance from the robot to the nest is more than 2 m and the robot is carrying an object

		mkConditions_Prep_ItemPhrase prep itemPhrase =
			-- mkAdv : Prep -> NP -> Adv | after 3 minutes
			SyntaxEng.mkAdv
				<(lin Prep prep) : Prep> -- after
				itemPhrase ; -- 3 minutes

	--------------------------------------------------------------------------------
	lincat -- ArenaAssumption
	
		ArenaAssumption = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- ArenaAssumption
	
		mkArenaAssumption_Statement statement =
			statement ; -- some locations of the arena except the source and the nest contain 1 obstacles
			
			

	--------------------------------------------------------------------------------
	lincat -- RobotAssumption
	
		RobotAssumption = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- RobotAssumption
	
		mkRobotAssumption_RobotStatement statement =
			statement ; -- the robot is a point mass
			


	--------------------------------------------------------------------------------
	lincat -- ElementAssumption
	
		ElementAssumption = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- ElementAssumption
	
		mkElementAssumption_Statement statement =
			statement ; -- the source has an x-width of 0.25 m and a y-width of 0.25 m			



	--------------------------------------------------------------------------------
	lincat -- InputEventMapping
	
		InputEventMapping = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- InputEventMapping

		mkInputEventMapping_InputSometimes str conditions =
			-- mkS : Cl -> S | the event obstacle occurs when the distance from the robot to an obstacle is less than 1 m
			mkS
				(
					-- mkCl : NP -> VP -> Cl | the event obstacle occurs when the distance from the robot to an obstacle is less than 1 m
					mkCl						
						(
							-- mkNP : Det -> CN -> NP | the event obstacle
							mkNP
								RoboWorldLexiconEng.the_Det -- the								
								(
									-- mkCN : N -> NP -> CN | event obstacle
									mkCN
										RoboWorldLexiconEng.event_N -- event										
										(
											-- symb : Str -> NP | obstacle
											symb
												str -- obstacle
										)
								)
						)							
						(
							-- mkVP: VP -> Adv -> VP | occurs when the distance from the robot to an obstacle is less than 1 m
							mkVP								
								(
									-- mkVP : V -> VP | occurs
									mkVP
										RoboWorldLexiconEng.occur_V -- occur
								)								
								conditions -- when the distance from the robot to an obstacle is less than 1 m
						)
				) ;	

		mkInputEventMapping_InputAlways str statements =
			-- mkS : Conj -> ListS -> S | the event angularSpeed is always available and it communicates the angular velocity of the robot
			mkS
				and_Conj -- and
				(
					-- mkListS : S -> S -> ListS | [the event angularSpeed is always available, it communicates the angular velocity of the robot]
					mkListS
						(
							-- mkS : Cl -> S | the event angularSpeed is always available
							mkS								
								(
									-- mkCl : NP -> VP -> Cl | the event angularSpeed is always available
									mkCl										
										(
											-- mkNP : Det -> CN -> NP | the event angularSpeed
											mkNP
												RoboWorldLexiconEng.the_Det -- the												
												(
													-- mkCN : N -> NP -> CN | event angularSpeed
													mkCN
														event_N -- event
														-- symb : Str -> NP | angularSpeed
														(
															symb str -- angularSpeed
														)
												)
										)										
										(
											-- mkVP : AdV -> VP -> VP | is always available
											mkVP
												always_AdV -- always												
												(
													-- mkVP : A -> VP | is available
													mkVP
														available_A -- available
												)
										)
								)
						)
						statements -- it communicates the angular velocity of the robot
				) ;

		mkInputEventMapping_InputNever str =
			-- mkS : Cl -> S | the event transferred never happens
			mkS				
				(
					-- mkCl : NP -> VP -> Cl | the event transferred never happens
					mkCl						
						(
							-- mkNP : Det -> CN -> NP | the event transferred
							mkNP
								RoboWorldLexiconEng.the_Det -- the								
								(
									-- mkCN : N -> NP -> CN | event transferred
									mkCN
										event_N -- event										
										(
											-- symb : Str -> NP | transferred
											symb
												str -- transferred
										)
								)
						)						
						(
							-- mkVP : AdV -> VP -> VP | never happens
							mkVP
								never_AdV -- never								
								(
									-- mkVP : V -> VP | happens
									mkVP
										happen_V -- happen
								)
						)
				) ;

	--------------------------------------------------------------------------------
	lincat -- OutputEventMapping
	
		OutputEventMapping = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- OutputEventMapping

		mkOutputEventMapping_OutputSometimes_NoConditions eventName statements =
			-- mkS : Adv -> S -> S | when the event takeoff occurs the velocity of the robot is set to 1 m/s upward
			mkS
				(
					-- mkAdv : Subj -> S -> Adv | when the event takeoff occurs
					SyntaxEng.mkAdv
						RoboWorldLexiconEng.when_Subj -- when
						(
							-- mkS : Cl -> S | the event takeoff occurs
							mkS
								(
									-- mkCl : NP -> VP -> Cl | the event takeoff occurs
									mkCl
										(
											-- mkNP : Det -> CN -> NP | the event takeoff
											mkNP
												RoboWorldLexiconEng.the_Det -- the
												(
													-- mkCN : N -> NP -> CN | event takeoff
													mkCN
														event_N -- event
														(
															-- symb : Str -> NP | takeoff
															symb
																eventName -- takeoff
														)
												)
										)
										occur_V -- occurs
								)
						)
			 	)
				statements ; -- the velocity of the robot is set to 1 m/s upward
				
		mkOutputEventMapping_DiagramaticOutput eventName val unit = 
			-- mkS : Cl -> S | the output event spray is defined by a diagram where one time unit is 1 s
			mkS
				(
					-- mkCL : NP -> VP -> S | the output event spray is defined by a diagram where one time unit is 1 s
					mkCl
						(
							-- mkNP : Det -> CN -> NP | the output event spray
							mkNP
								RoboWorldLexiconEng.the_Det -- the
								(
									-- mkCN : N -> NP -> CN | output event spray
									mkCN
										RoboWorldLexiconEng.output_N -- output
										(
											-- mkNP : CN -> NP | event spray
											mkNP
											(
												-- mkCN : N -> NP -> CN | event spray
												(
													mkCN
														RoboWorldLexiconEng.event_N
														(
															-- symb : Str -> NP | spray
															symb
																eventName -- turnBack()													
														)
												)
											)
										)
								)
						)
						(
							-- mkVP : VP -> Adv -> VP | is defined by a diagram where one time unit is 1 s
							mkVP
								(						
									-- passiveVP : V2 -> NP -> VP | is defined by a diagram
									passiveVP
										<(lin V2 define_V2) : V2> -- define
										<(lin NP
											(
												-- mkItemPhrase_DeterminedIP : Cat.Det -> Item -> ItemPhrase | a diagram
												mkItemPhrase_DeterminedIP
													RoboWorldLexiconEng.a_Det -- a
													(
														-- mkItem_from_BasicItem : BasicItem -> Item | diagram
														mkItem_from_BasicItem
															(
																-- mkBasicItem_single_noun : Cat.N -> BasicItem
																mkBasicItem_single_noun
																	diagram_N
															)
													)
											)) : NP>
								)
								(
									-- mkAdv : Subj -> S -> Adv | where one time unit is 1 s
									SyntaxEng.mkAdv
										where_Subj -- where
										(
											-- mkS : Cl -> S | one time unit is 1 s
											mkS
												(
													-- mkCl : NP -> VP -> Cl | one time unit is 1 s
													mkCl
														(
															-- mkNP : Det -> CN -> NP | one time unit
															mkNP
																(
																	-- mkDet : Numeral -> Det | one
																	mkDet
																	(
																		-- mkNumeral : Unit -> Numeral | one
																		mkNumeral
																			n1_Unit -- one
																	)
																)
																(
																	-- mkCN : N -> NP -> CN | time unit
																	mkCN
																		time_N -- time
																		(
																			-- mkNP : CN -> NP | unit
																			mkNP
																				(
																					-- mkCN : N -> CN | unit
																					mkCN
																						unit_1_N -- unit
																				)
																		)
																)
														)
								 	   					(
								 	   						-- mkVP : Comp -> VP | is 1 s
								 	   						mkVP
								 	   							(
								 	   								-- mkComp : NP -> Comp | is 1 s
								 	   								mkComp
								 	   									<(lin NP
									 	   									(
									 	   										-- mkItemPhrase_QuantifiedIP_with_float : Float -> Item -> ItemPhrase | 1 s
										 	   									mkItemPhrase_QuantifiedIP_with_float
										 	   										val -- 1
										 	   										(
										 	   											-- mkItem_from_BasicItem : BasicItem -> Item | s
										 	   											mkItem_from_BasicItem
										 	   												(
										 	   													-- mkBasicItem_Unit : Unit -> BasicItem | s
										 	   													mkBasicItem_Unit
										 	   														unit -- s
										 	   												)
										 	   										)
									 	   									)) : NP>
								 	   							)
								 	   					)								
								 				)
										)
						 		)
						)
				) ;				

	--------------------------------------------------------------------------------
	lincat -- OperationMapping
	
		OperationMapping = CatEng.S ;
		
	--------------------------------------------------------------------------------
	lin -- OperationMapping
		
		mkOperationMapping_OutputSometimes_NoConditions opCall statements =
			-- mkS : Adv -> S -> S | when the operation move(ls,as) is called
			--                       the velocity of the robot is set to ls m/s towards the orientation of the robot
			--                       and the angular velocity of the robot is set to as rad/s
			mkS
				(
					-- mkAdv : Subj -> S -> Adv | when the operation move(ls,as) is called
					SyntaxEng.mkAdv
						RoboWorldLexiconEng.when_Subj -- when
						(
							-- mkS : Cl -> S | the operation move(ls,as) is called
							mkS
							(
								-- mkCl : NP -> VP -> Cl | the operation move(ls,as) is called
								mkCl
									(
										-- mkNP : Det -> CN -> NP | the operation move(ls,as)
										mkNP
											RoboWorldLexiconEng.the_Det -- the
											(
												-- mkCN : N -> NP -> CN | operation move(ls,as)
												mkCN
													RoboWorldLexiconEng.operation_N -- operation
													(
														-- symb : Str -> NP | operation
														symb
															opCall -- operation
													)
											)
									)									
									(
										-- passiveVP : V2 -> VP | is called
										passiveVP
											RoboWorldLexiconEng.call_V2 -- call
									)
							)
						)
				)
				statements ; -- the velocity of the robot is set to ls m/s towards the orientation of the robot
				             -- and the angular velocity of the robot is set to as rad/s		

		mkOperationMapping_OutputSometimes_WithConditions opCall conditions statements =
			-- mkS : Adv -> S -> S | when the operation Store() is called as soon as the distance from the robot to the source is less than 1 m the robot places an object in the nest
			mkS
				(
					-- mkAdv : Subj -> S -> Adv | when the operation Store() is called
					SyntaxEng.mkAdv
						RoboWorldLexiconEng.when_Subj -- when
						(
							-- mkS : Cl -> S | the operation Store() is called
							mkS (
								-- mkCl : NP -> VP -> Cl | the operation Store() is called
								mkCl
									(
										-- mkNP : Det -> CN -> NP | the operation Store()
										mkNP
											RoboWorldLexiconEng.the_Det -- the
											(
												-- mkCN : N -> NP -> CN | operation Store()
												mkCN
													operation_N -- operation
													(
														-- symb : Str -> NP | Store()
														symb
															opCall -- Store()
													)
											)
									)
									(
										-- passiveVP : V2 -> VP | is called
										passiveVP
											RoboWorldLexiconEng.call_V2 -- call
									)
							)
						)
				)
				(
					-- mkS : Adv -> S -> S | as soon as the distance from the robot to the source is less than 1 m the robot places an object in the nest
					mkS
						conditions -- as soon as the distance from the robot to the source is less than 1 m
						statements -- the robot places an object in the nest
				) ;
									
		mkOperationMapping_NoOutput opCall =
			-- mkS : Adv -> S -> S | when the operation Transfer() is called nothing happens
			mkS
				(
					-- mkAdv : Subj -> S -> Adv | when the operation Transfer() is called
					SyntaxEng.mkAdv
						RoboWorldLexiconEng.when_Subj -- when
						(
							-- mkS : Cl -> S | the operation Transfer() is called
							mkS
								(
									-- mkCl : NP -> VP -> S | the operation Transfer() is called
									mkCl
										(
											-- mkNP : Det -> CN -> NP | the operation Transfer()
											mkNP
												RoboWorldLexiconEng.the_Det -- the
												(
													-- mkCN : N -> NP -> CN | operation Transfer()
													mkCN
														operation_N -- operation
														(
															-- symb : Str -> NP | Transfer()
															symb
																opCall -- Transfer()
														)
												)
										)
										(
											-- passiveVP : V2 -> VP | is called
											passiveVP
												call_V2 -- call
										)
								)
						)
				)
				(
					-- mkS : Cl -> S | nothing happens
					mkS
						(
							-- mkCl : NP -> VP -> Cl
							mkCl
								nothing_NP -- nothing
								(
									-- mkVP : V -> VP | happens
									mkVP
										happen_V -- happen
								)
						)
				) ;																		
									
		mkOperationMapping_DiagrammaticOutput opCall val unit = 
			-- mkS : Cl -> S | the operation turnBack() is defined by a diagram where one time unit is 1 s
			mkS
				(
					-- mkCL : NP -> VP -> S | the operation turnBack() is defined by a diagram where one time unit is 1 s
					mkCl
						(
							-- mkNP : Det -> CN -> NP | the operation turnBack()
							mkNP
								RoboWorldLexiconEng.the_Det -- the
								(
									-- mkCN : N -> NP -> CN | operation turnBack()
									mkCN
										operation_N -- operation
										(
											-- symb : Str -> NP | turnBack()
											symb
												opCall -- turnBack()
										)
								)
						)
						(
							-- mkVP : VP -> Adv -> VP | is defined by a diagram where one time unit is 1 s
							mkVP
								(						
									-- passiveVP : V2 -> NP -> VP | is defined by a diagram
									passiveVP
										<(lin V2 define_V2) : V2> -- define
										<(lin NP
											(
												-- mkItemPhrase_DeterminedIP : Cat.Det -> Item -> ItemPhrase | a diagram
												mkItemPhrase_DeterminedIP
													RoboWorldLexiconEng.a_Det -- a
													(
														-- mkItem_from_BasicItem : BasicItem -> Item | diagram
														mkItem_from_BasicItem
															(
																-- mkBasicItem_single_noun : Cat.N -> BasicItem
																mkBasicItem_single_noun
																	diagram_N
															)
													)
											)) : NP>
								)
								(
									-- mkAdv : Subj -> S -> Adv | where one time unit is 1 s
									SyntaxEng.mkAdv
										where_Subj -- where
										(
											-- mkS : Cl -> S | one time unit is 1 s
											mkS
												(
													-- mkCl : NP -> VP -> Cl | one time unit is 1 s
													mkCl
														(
															-- mkNP : Det -> CN -> NP | one time unit
															mkNP
																(
																	-- mkDet : Numeral -> Det | one
																	mkDet
																	(
																		-- mkNumeral : Unit -> Numeral | one
																		mkNumeral
																			n1_Unit -- one
																	)
																)
																(
																	-- mkCN : N -> NP -> CN | time unit
																	mkCN
																		time_N -- time
																		(
																			-- mkNP : CN -> NP | unit
																			mkNP
																				(
																					-- mkCN : N -> CN | unit
																					mkCN
																						unit_1_N -- unit
																				)
																		)
																)
														)
								 	   					(
								 	   						-- mkVP : Comp -> VP | is 1 s
								 	   						mkVP
								 	   							(
								 	   								-- mkComp : NP -> Comp | is 1 s
								 	   								mkComp
								 	   									<(lin NP
									 	   									(
									 	   										-- mkItemPhrase_QuantifiedIP_with_float : Float -> Item -> ItemPhrase | 1 s
										 	   									mkItemPhrase_QuantifiedIP_with_float
										 	   										val -- 1
										 	   										(
										 	   											-- mkItem_from_BasicItem : BasicItem -> Item | s
										 	   											mkItem_from_BasicItem
										 	   												(
										 	   													-- mkBasicItem_Unit : Unit -> BasicItem | s
										 	   													mkBasicItem_Unit
										 	   														unit -- s
										 	   												)
										 	   										)
									 	   									)) : NP>
								 	   							)
								 	   					)								
								 				)
										)
						 		)
						)
				) ;

	--------------------------------------------------------------------------------
	lincat -- VariableMapping
	
		VariableMapping = CatEng.S ;
		
	--------------------------------------------------------------------------------
	--fun -- VariableMapping

	
	
	--------------------------------------------------------------------------------
	-- SPECIAL WORDS (HELP FUNCTION)
	lin _special_N = mkN "[N]" "[N]";
	lin _special_A = mkA "[A]" "[A]";
	lin _special_Adv = ParadigmsEng.mkAdv "[Adv]";
	lin _special_Prep = mkPrep "[Prep]";
	lin _special_Pron = MorphoEng.mkPron "[Pron]" "[Pron]" "[Pron]" "[Pron]" singular P3 nonhuman;
	--lin _special_Det = ?;
	lin _special_V2 = mkV2 (mkV "[V2]" "[V2]" "[V2]" "[V2]" "[V2]");
	lin _special_Subj = mkSubj "[Subj]";
	
	lin _special_empty_V = mkV "" "" "" "" "";	
	lin _special_Unit = mkN "[Unit]" "[Unit]";
	lin _special_BasicItem = mkCN (mkN "[BasicItem]" "[BasicItem]");
	lin _special_CompoundItem = mkCN (mkN "[CompoundItem]" "[CompoundItem]");
	lin _special_Item = mkCN (mkN "[Item]" "[Item]");
	lin _special_ItemPhrase = mkNP (mkN "[ItemPhrase]" "[ItemPhrase]");
	lin _special_ItemPhraseList = mkListNP (mkNP (mkN "[ItemPhrase]" "[ItemPhrase]")) (mkNP (mkN "[ItemPhrase]" "[ItemPhrase]"));
	lin _special_Statement = mkS (mkCl (mkNP (mkN "[Statement]" "[Statement]")) _special_empty_V);
	lin _special_StatementList = mkListS (mkS (mkCl (mkNP (mkN "[StatementList]" "[StatementList]")) _special_empty_V)) (mkS (mkCl (mkNP (mkN "[StatementList]" "[StatementList]")) _special_empty_V));
	lin _special_Statements = mkS (mkCl (mkNP (mkN "[Statements]" "[Statements]")) _special_empty_V);
	lin _special_Conditions = ParadigmsEng.mkAdv "[Conditions]";
	lin _special_ArenaAssumption = mkS (mkCl (mkNP (mkN "[ArenaAssumption]" "[ArenaAssumption]")) _special_empty_V);
	lin _special_RobotAssumption = mkS (mkCl (mkNP (mkN "[RobotAssumption]" "[RobotAssumption]")) _special_empty_V);
	lin _special_ElementAssumption = mkS (mkCl (mkNP (mkN "[ElementAssumption]" "[ElementAssumption]")) _special_empty_V);
	lin _special_InputEventMapping = mkS (mkCl (mkNP (mkN "[InputEventMapping]" "[InputEventMapping]")) _special_empty_V);
	lin _special_OutputEventMapping = mkS (mkCl (mkNP (mkN "[OutputEventMapping]" "[OutputEventMapping]")) _special_empty_V);
	lin _special_OperationMapping = mkS (mkCl (mkNP (mkN "[OperationMapping]" "[OperationMapping]")) _special_empty_V);
	lin _special_VariableMapping = mkS (mkCl (mkNP (mkN "[VariableMapping]" "[VariableMapping]")) _special_empty_V);
	
}
